{{Walker.spin}}

CON

_CLKMODE         = XTAL1 + PLL16X
_XINFREQ         = 5_000_000


'Constants for using the Propeller Servo Control (PSC) boards    
COMPIN           = 23                'Pin used for communication with the PSC
PSC_BAUD         = 0                 'Baud rate (0 - 2400, 1 - 38400)
Ramp             = 40                'Ramp is the speed between 0-63 that the PSC turns the servos  (fast to slow)
Ramp2            = 60 
HipSpeedMultiplier =3

'Constants for the leg servos
RPawFoot        = 31
RPawKnee        = 30
RPawHip         = 29

RFFoot          = 26
RFKnee          = 27
RFHip           = 28

RRFoot          = 23
RRKnee          = 22
RRHip           = 21

LPawFoot        =  0
LPawKnee        =  1
LPawHip         =  2

LFFoot          = 15
LFKnee          = 14
LFHip           = 13

LRFoot          =  3
LRKnee          =  4
LRHip           =  5

'The maximune height that the knees can go up
KneeHightMax    =  1000
KneeHightMin    =   400

'The maximune height that the three down legs can go in an attempt to raise
'the body over an obstical
BodyHeightMax   =  550
BodyHeightMin   =  950

'Toe Sensor Pins
RPawToeSensor   =  0
RFToeSensor     =  1
RRToeSensor     =  2
LPawToeSensor   =  5
LFToeSensor     =  4
LRToeSensor     =  3


VAR
'Ping))) variables
Long                  range            'Distance for Ping)))
Long                  PingServoD       'Direction/PW of the servo holding the Ping)))
Long                  PingServoR
Long                  PingServoL
Long                  DirectionPW
Long                  PingDirection

'Long                  MasterDirection
'Long                  LegNumber
'Long                  TotalAmountOfMovementInOneTurn

Long                  LegGroup

Long                  HipEndpoint
Long                  HipStartPoint
Long                  FootEndpoint
Long                  FootStartPoint

'"A" set of legs
Long                  HipEndpointLPaw
Long                  HipStartPointLPaw
Long                  FootEndpointLPaw
Long                  FootStartPointLPaw

Long                  HipEndpointRF
Long                  HipStartPointRF
Long                  FootEndpointRF
Long                  FootStartpointRF

Long                  HipEndpointLR
Long                  HipStartPointLR
Long                  FootEndpointLR
Long                  FootStartPointLR

'"B" set of legs
Long                  HipEndpointRPaw
Long                  HipStartPointRPaw
Long                  FootEndpointRPaw
Long                  FootStartPointRPaw

Long                  HipEndpointLF
Long                  HipStartPointLF
Long                  FootEndpointLF
Long                  FootStartpointLF

Long                  HipEndpointRR
Long                  HipStartPointRR
Long                  FootEndpointRR
Long                  FootStartPointRR




'Variables for the PW of each leg servo
Long                  RPawFootPW        
Long                  RPawKneePW        
Long                  RPawHipPW         

Long                  RFFootPW          
Long                  RFKneePW          
Long                  RFHipPW           

Long                  RRFootPW          
Long                  RRKneePW          
Long                  RRHipPW           

Long                  LPawFootPW        
Long                  LPawKneePW        
Long                  LPawHipPW         

Long                  LFFootPW          
Long                  LFKneePW          
Long                  LFHipPW           

Long                  LRFootPW          
Long                  LRKneePW          
Long                  LRHipPW            

Long                  RPawFtSign
Long                  RFFtSign
Long                  RRFtSign

Long                  PW



Long                  BodyHeight


Long                  HipLegStateLPaw
Long                  HipLegStateRF
Long                  HipLegStateLR

Long                  HipDirectionLPaw
Long                  HipDirectionRF
Long                  HipDirectionLR

Long                  HipPositionLPaw
Long                  HipPositionRF
Long                  HipPositionLR

Long                  HipsGroup0

Long                  FootState0
Long                  FootLegStateLPaw
Long                  FootLegStateRF
Long                  FootLegStateLR

Long                  FootDirectionLPaw
Long                  FootDirectionRF
Long                  FootDirectionLR

Long                  FootPositionLPaw
Long                  FootPositionRF
Long                  FootPositionLR

 
Long                  TotalHipMovement

OBJ
     PSC   : "ServoControllerSerial"
    'ping   : "ping"
     Debug   : "FullDuplexSerialPlus"
     Fstring : "FloatString"
     F       : "Float32Full"  


PUB StartUp
''Starts up and Initializes leg values from defult "0" to 750
    waitcnt(clkfreq*5 + cnt)
    
    PSC.START(COMPIN, PSC_BAUD) 
    Debug.Start(31, 30, 0, 57600)
    Debug.tx(Debug#CLS) 
    Debug.str(string(13, "Starting! :) "))
    Debug.str(string(13, "Initializing leg values! :) "))

    RPawFootPW     := 750   
    RPawKneePW     := 750   
    RPawHipPW      := 750   

    RFFootPW       := 750   
    RFKneePW       := 750   
    RFHipPW        := 750   

    RRFootPW       := 750   
    RRKneePW       := 750   
    RRHipPW        := 750   

    LPawFootPW     := 750   
    LPawKneePW     := 750   
    LPawHipPW      := 750   

    LFFootPW       := 750   
    LFKneePW       := 750   
    LFHipPW        := 750   

    LRFootPW       := 750   
    LRKneePW       := 750   
    LRHipPW        := 750


    BodyHeight     := 750

    PingDirection  := 90  '<--- in degrees the direction the ping is looking; for debuging only
    
    waitcnt(clkfreq*1 + cnt)
    'LegSetUp
    LegValues

PUB LegValues                               
'Caculates the start and end values of the hips and feet
Debug.str(string(13, "LegValues LegValues LegValues LegValues LegValues LegValues LegValues"))
waitcnt(clkfreq*1 + cnt)

 IF LegGroup == 0
     Debug.str(string(13, "LegValues group 0"))

    CaculateLegValues(PingDirection, 4, 150)           'Caculate the leg values (master direction, leg number used for 
                                                       'caculating the angle correction due to the leg's location
                                                       'relative to the "home leg")
    HipEndpointLPaw    := HipEndpoint
    HipStartPointLPaw  := HipStartPoint
    FootEndpointLPaw   := FootEndpoint
    FootStartPointLPaw := FootStartPoint

    CaculateLegValues(PingDirection, 0, 150)

    HipEndpointRF     := HipEndpoint
    HipStartPointRF   := HipStartPoint
    FootEndpointRF    := FootEndpoint
    FootStartPointRF  := FootStartPoint

    CaculateLegValues(PingDirection, 2, 150)
    HipEndpointLR     := HipEndpoint
    HipStartPointLR   := HipStartPoint
    FootEndpointLR    := FootEndpoint
    FootStartPointLR  := FootStartPoint
    
 
    'waitcnt(clkfreq*6 + cnt)        '<--- for debugging and allowing time to check values******************
    Group0

 ELSEIF LegGroup :=1

    Debug.str(string(13, "LegValues Leg group 1 "))


   CaculateLegValues(PingDirection, 5, 150)           'Caculate the leg values (master direction, leg number used for 
                                                       'caculating the angle correction due to the leg's location
                                                       'relative to the "home leg")   
    HipEndpointRPaw    := HipEndpoint
    HipStartPointRPaw  := HipStartPoint
    FootEndpointRPaw   := FootEndpoint
    FootStartPointRPaw := FootStartPoint

    CaculateLegValues(PingDirection, 3, 150)

    HipEndpointLF     := HipEndpoint
    HipStartPointLF   := HipStartPoint
    FootEndpointLF    := FootEndpoint
    FootStartPointLF  := FootStartPoint

    CaculateLegValues(PingDirection, 1, 150)
    HipEndpointRR     := HipEndpoint
    HipStartPointRR   := HipStartPoint
    FootEndpointRR    := FootEndpoint
    FootStartPointRR  := FootStartPoint


    'waitcnt(clkfreq*6 + cnt)        '<--- for debugging and allowing time to check values********************

    Group1    

PUB  Group0    
    Debug.str(string(13, "Group 0 "))
''Caculating if you need to add, subtract or do nothing with moving the hip and foot from
''it's starting position

HipLegStateLPaw  := 0
HipLegStateRF    := 0
HipLegStateLR    := 0
HipsGroup0       := 0

  If      HipStartpointLPaw == HipEndpointLPaw
          HipDirectionLPaw := 0
          HipLegStateLPaw  := 1
  ELSEIf  HipStartpointLPaw => HipEndpointLPaw
          HipDirectionLPaw := -1                    
  ELSEIf  HipStartpointLPaw =< HipEndpointLPaw
          HipDirectionLPaw := 1                     

  If      HipStartPointRF == HipEndPointRF                    
          HipDirectionRF := 0
          HipLegStateRF  := 1
  ElseIf  HipStartPointRF => HipEndPointRF
          HipDirectionRF := -1
  ElseIf  HipStartPointRF =< HipEndPointRF
          HipDirectionRF := 1

  If      HipStartpointLR == HipEndpointLR
          HipDirectionLR := 0
          HipLegStateLR  := 1
  ElseIf  HipStartpointLR => HipEndpointLR
          HipDirectionLR := -1                      
  ElseIf  HipStartpointLR =< HipEndpointLR
          HipDirectionLR :=  1                       

HipPositionLPaw := HipStartpointLPaw
HipPositionRF   := HipStartPointRF
HipPositionLR   := HipStartpointLR
    
    
 MoveHipsGroup0


PUB MoveHipsGroup0
    Debug.str(string(13, "MoveHipsGroup0 "))
    Debug.Bin(ina[0..5], 6)                           ''Displays the state of the 6 leg touch
                                                      ''sensors.  A "1" means that it's touching

    Debug.str(string(13, "HipLegStateLPaw "))
    Debug.dec(HipLegStateLPaw)
    Debug.str(string(13, "HipLegStateRF "))
    Debug.dec(HipLegStateRF)
    Debug.str(string(13, "HipLegStateLR "))
    Debug.dec(HipLegStateLR)
    Debug.str(string(13, "HipsGroup0 "))
    Debug.dec(HipsGroup0)

    HipsGroup0 := HipLegStateLPaw + HipLegStateRF + HipLegStateLR 
    If HipsGroup0 == 3                                '+=+=+=+***** change "2" to "3" (1 used for debugging)
       Debug.str(string(13, "Hip group 0 are in place; now going to lower feet in group 0! :) ")) 
       MoveFeetGroup0
    
                                                      
    MoveHipLPaw  

PUB MoveHipLPaw   'Group 0
    Debug.str(string(13, "MoveHipLPaw "))

    If HipPositionLPaw == HipEndpointLPaw
       HipLegStateLPaw  := 1
       Debug.str(string(13, "Hip LPaw is done moving; skipping LPaw to MoveHipRF! :) "))
       MoveHipRF

    HipPositionLPaw := HipPositionLPaw + HipDirectionLPaw*HipSpeedMultiplier
    IF HipDirectionLPaw == 1
       HipPositionLPaw <#= HipEndpointLPaw
       Debug.str(string(13, "HipPositionLPaw <#= HipEndpointLPaw Limit Maximun"))
    IF HipDirectionLPaw == -1
       HipPositionLPaw #>= HipEndpointLPaw
       Debug.str(string(13, "HipPositionLPaw #>= HipEndpointLPaw Limit Minimun"))
       
    PSC.SETPOS(LPawHip, Ramp, HipPositionLPaw)
    'waitcnt(clkfreq/10 + cnt)
    
    ''This section checks to see if the Toe sensor is touching something and if it is,
    ''pulls the hip back a bit and raises the leg
    If ina[LPawToeSensor] == 1
      Debug.str(string(13, "Leg/pin LPawToeSensor touching!!! :) "))
      HipPositionLPaw := HipPositionLPaw + (HipDirectionLPaw* -10)
      LPawKneePW := LPawKneePW + 15                                               
      LPawKneePW <#= KneeHightMax
      Debug.str(string(13, "LPawKneePW "))
      Debug.dec(LPawKneePW) 
     'Lift body a little bit
      BodyHeight := BodyHeight - 15     
      Debug.str(string(13, "BodyHeight"))
      Debug.dec(BodyHeight)
     
      If BodyHeight =< BodyHeightMax
           BodyHeight := BodyHeight - 15 
           Debug.str(string(13, "BackUpAndChangeApproach")) 'Going to back-up and change directions         
           BackUpAndChangeApproach 
      IF LPawKneePW => KneeHightMax
           LPawKneePW := LPawKneePW - 15    
           Debug.str(string(13, "BackUpAndChangeApproach becasue of LPawKneePW")) 'Going to back-up and change directions         
           BackUpAndChangeApproach 
      'waitcnt(clkfreq/10 + cnt)
       
     'Lifting legs and raising body  
      Debug.str(string(13, "Raising Body"))
      PSC.SETPOS(RPawKnee, Ramp2, BodyHeight)
      PSC.SETPOS(RPawFoot, Ramp2, BodyHeight)

      PSC.SETPOS(LFKnee, Ramp2, BodyHeight)
      PSC.SETPOS(LFFoot, Ramp2, BodyHeight)

      PSC.SETPOS(RRKnee, Ramp2, BodyHeight)
      PSC.SETPOS(RRFoot, Ramp2, BodyHeight)  
        
      'Moves leg back and up
      PSC.SETPOS(LPawHip, Ramp, HipPositionLPaw)       
      PSC.SETPOS(LPawKnee, Ramp, LPawKneePW)
      PSC.SETPOS(LPawFoot, Ramp, LPawKneePW)  
      'waitcnt(clkfreq/10 + cnt)

    MoveHipRF        

PUB MoveHipRF     'Group 0
    Debug.str(string(13, "MoveHipRF"))

    If HipPositionRF == HipEndPointRF
       HipLegStateRF  := 1
       Debug.str(string(13, "Hip RF is done moving; skipping RF to MoveHipLR! :) "))
       MoveHipLR

    HipPositionRF := HipPositionRF + HipDirectionRF*HipSpeedMultiplier
    IF HipDirectionRF == 1
       HipPositionRF <#= HipEndPointRF                            
       Debug.str(string(13, "HipPositionRF <#= HipEndPointRF"))
    IF HipDirectionRF == -1
       HipPositionRF #>= HipEndPointRF                           
       Debug.str(string(13, "HipPositionRF #>= HipEndPointRF"))
       
    PSC.SETPOS(RFHip, Ramp, HipPositionRF)
    'waitcnt(clkfreq/10 + cnt)
    
    ''This section checks to see if the Toe sensor is touching something and if it is,
    ''pulls the hip back a bit and raises the leg
    If ina[RFToeSensor] == 1
      Debug.str(string(13, "Leg/pin RFToeSensor touching!!! :) "))
      HipPositionRF := HipPositionRF + (HipDirectionRF* -10)
      RFKneePW := RFKneePW + 15
      RFKneePW <#= KneeHightMax
      Debug.str(string(13, "RFKneePW "))
      Debug.dec(RFKneePW) 
     'Lift body a little bit
      BodyHeight := BodyHeight - 15     
      Debug.str(string(13, "BodyHeight"))
      Debug.dec(BodyHeight)
     
      If BodyHeight =< BodyHeightMax
           BodyHeight := BodyHeight - 15
           Debug.str(string(13, "BackUpAndChangeApproach")) 'Going to back-up and change directions         
           BackUpAndChangeApproach 
      IF RFKneePW => KneeHightMax
           RFKneePW := RFKneePW - 15   
           Debug.str(string(13, "BackUpAndChangeApproach")) 'Going to back-up and change directions         
           BackUpAndChangeApproach 
      'waitcnt(clkfreq/10 + cnt)

     'Lifting legs and raising body  
      Debug.str(string(13, "Raising Body"))
      PSC.SETPOS(RPawKnee, Ramp2, BodyHeight)
      PSC.SETPOS(RPawFoot, Ramp2, BodyHeight)

      PSC.SETPOS(LFKnee, Ramp2, BodyHeight)
      PSC.SETPOS(LFFoot, Ramp2, BodyHeight)

      PSC.SETPOS(RRKnee, Ramp2, BodyHeight)
      PSC.SETPOS(RRFoot, Ramp2, BodyHeight)  
        
      'Moves leg back and up
      PSC.SETPOS(RFHip, Ramp, HipPositionRF)       
      PSC.SETPOS(RFKnee, Ramp, RFKneePW)
      PSC.SETPOS(RFFoot, Ramp, RFKneePW)  
   'waitcnt(clkfreq/10 + cnt)    

    MoveHipLR 
 
PUB MoveHipLR 
    Debug.str(string(13, "MoveHipLR"))
    'waitcnt(clkfreq/1 + cnt)

    If HipPositionLR == HipEndpointLR
       HipLegStateLR  := 1
       Debug.str(string(13, "Hip LR is done moving; skipping LR to MoveHipsGroup0! :) "))
       MoveHipsGroup0

    HipPositionLR := HipPositionLR + HipDirectionLR*HipSpeedMultiplier
    IF HipDirectionLR == 1
       HipPositionLR <#= HipEndpointLR
       Debug.str(string(13, "HipPositionLR <#= HipEndpointLR"))
    IF HipDirectionLR == -1
       HipPositionLR #>= HipEndpointLR
       Debug.str(string(13, "HipPositionLR #>= HipEndpointLR "))
       
    PSC.SETPOS(LRHip, Ramp, HipPositionLR)
    'waitcnt(clkfreq/10 + cnt)
    
    ''This section checks to see if the Toe sensor is touching something and if it is,
    ''pulls the hip back a bit and raises the leg
    If ina[LRToeSensor] == 1
      Debug.str(string(13, "Leg/pin LRToeSensor touching!!! :) "))
      HipPositionLR := HipPositionLR + (HipDirectionLR* -10)
      LRKneePW := LRKneePW + 15
      LRKneePW <#= KneeHightMax
      Debug.str(string(13, "LRKneePW "))
      Debug.dec(LRKneePW) 
     'Lift body a little bit
      BodyHeight := BodyHeight - 15     
      Debug.str(string(13, "BodyHeight"))
      Debug.dec(BodyHeight)
     
      If BodyHeight =< BodyHeightMax
           BodyHeight := BodyHeight - 15
           Debug.str(string(13, "BackUpAndChangeApproach")) 'Going to back-up and change directions         
           BackUpAndChangeApproach 
      IF LRKneePW => KneeHightMax
           LRKneePW := LRKneePW - 15     
           Debug.str(string(13, "BackUpAndChangeApproach  LRKneePW")) 'Going to back-up and change directions         
           BackUpAndChangeApproach 
      'waitcnt(clkfreq/10 + cnt)
       
      Debug.str(string(13, "Raising Body"))
      PSC.SETPOS(RPawKnee, Ramp2, BodyHeight)
      PSC.SETPOS(RPawFoot, Ramp2, BodyHeight)

      PSC.SETPOS(LFKnee, Ramp2, BodyHeight)
      PSC.SETPOS(LFFoot, Ramp2, BodyHeight)

      PSC.SETPOS(RRKnee, Ramp2, BodyHeight)
      PSC.SETPOS(RRFoot, Ramp2, BodyHeight)  
       
      'Moves leg back and up
      PSC.SETPOS(LRHip, Ramp, HipPositionLR)       
      PSC.SETPOS(LRKnee, Ramp, LRKneePW)
      PSC.SETPOS(LRFoot, Ramp, LRKneePW)  
  waitcnt(clkfreq/10 + cnt)

  
  MoveHipsGroup0



PUB MoveFeetGroup0

''Determins if feet are adding or subtracting
    Debug.str(string(13, "MoveFeetGroup0")) 


FootState0       :=  0
FootLegStateLPaw :=  0
FootLegStateRF   :=  0
FootLegStateLR   :=  0
 

'Foot states
    If FootStartPointLPaw == FootEndpointLPaw
       FootDirectionLPaw := 0
       FootLegStateLPaw  := 1
    ELSEIf FootStartPointLPaw => FootEndpointLPaw
       FootDirectionLPaw := -1
    ELSEIf FootStartPointLPaw =< FootEndpointLPaw
       FootDirectionLPaw := 1

    If FootStartPointRF == FootEndpointRF
       FootDirectionRF := 0
       FootLegStateRF  := 1
    ElseIf FootStartPointRF => FootEndpointRF
       FootDirectionRF := -1
    ElseIf FootStartPointRF =< FootEndpointRF
       FootDirectionRF := 1

    If FootStartPointLR == FootEndpointLR
       FootDirectionLR := 0
       FootLegStateLR  := 1
    ElseIf FootStartPointLR => FootEndpointLR
       FootDirectionLR := -1
    ElseIf FootStartPointLR =< FootEndpointLR
       FootDirectionLR := 1

FootPositionLPaw := FootStartpointLPaw
FootPositionRF   := FootStartPointRF
FootPositionLR   := FootStartpointLR



MoveFootGroup0

PUB MoveFootGroup0
''Moves (extends or retracts) feet to starting postions
    Debug.str(string(13, "MoveFootGroup0")) 

    Debug.Bin(ina[0..5], 6)                           ''Displays the state of the 6 leg touch
                                                      ''sensors.  A "1" means that it's touching

    Debug.str(string(13, "FootLegStateLPaw "))
    Debug.dec(FootLegStateLPaw)
    Debug.str(string(13, "FootLegStateRF "))
    Debug.dec(FootLegStateRF)
    Debug.str(string(13, "FootLegStateLR "))
    Debug.dec(FootLegStateLR)
    Debug.str(string(13, "FootState0 "))
    Debug.dec(FootState0)

    FootState0 := FootLegStateLPaw + FootLegStateRF + FootLegStateLR
    If FootState0 == 3                                '+=+=+=+***** change "2" to "3" (1 used for debugging)
       Debug.str(string(13, "Foot group 0 is in place; now going to lower foot group 0! :) ")) 
       SetFootLowerFootStatesGroup0
    
                                                      
MoveFootLPaw



PUB MoveFootLPaw
''Moves MoveFootLPaw to it's starting position
    Debug.str(string(13, "MoveFootLPaw")) 

    If FootPositionLPaw == FootEndpointLPaw
       FootLegStateLPaw  := 1
       Debug.str(string(13, "Foot LPaw is done moving; skipping LPaw to FootRF! :) "))
       MoveFootRF

    FootPositionLPaw := FootPositionLPaw + FootDirectionLPaw*HipSpeedMultiplier
      Debug.str(string(13, "FootPositionLPaw"))
      Debug.dec(FootPositionLPaw)


    IF FootDirectionLPaw == 1
       FootPositionLPaw <#= FootEndpointLPaw
       Debug.str(string(13, "FootLPawPlacement <#= FootFinishLPaw"))
    IF FootDirectionLPaw == -1
       FootPositionLPaw #>= FootEndpointLPaw
       Debug.str(string(13, "FootLPawPlacement #>= FootFinishLPaw"))
       
    PSC.SETPOS(LPawFoot, Ramp, FootPositionLPaw)
    'waitcnt(clkfreq/10 + cnt)
    
    ''This section checks to see if the Toe sensor is touching something and if it is,
    ''pulls the Foot back a bit and raises the leg
    If ina[LPawToeSensor] == 1
      Debug.str(string(13, "Leg/pin LPawToeSensor touching!!! :) "))
      FootPositionLPaw := FootPositionLPaw + (FootDirectionLPaw* -10)
      LPawKneePW := LPawKneePW + 15
      LPawKneePW <#= KneeHightMax
      Debug.str(string(13, "LPawKneePW "))
      Debug.dec(LPawKneePW) 
     'Lift body a little bit
      BodyHeight := BodyHeight - 15     
      Debug.str(string(13, "BodyHeight"))
      Debug.dec(BodyHeight)
     
      If BodyHeight =< BodyHeightMax
           BodyHeight := BodyHeight - 15
           Debug.str(string(13, "BackUpAndChangeApproach")) 'Going to back-up and change directions         
           BackUpAndChangeApproach 
      IF LPawKneePW => KneeHightMax
           LPawKneePW := LPawKneePW - 15          
           Debug.str(string(13, "BackUpAndChangeApproach because of  KneeHightMax")) 'Going to back-up and change directions         
           BackUpAndChangeApproach 
      'waitcnt(clkfreq/10 + cnt)
       
     'Lifting legs and raising body  
      Debug.str(string(13, "Raising Body"))
      PSC.SETPOS(RPawKnee, Ramp2, BodyHeight)
      PSC.SETPOS(RPawFoot, Ramp2, BodyHeight)

      PSC.SETPOS(LFKnee, Ramp2, BodyHeight)
      PSC.SETPOS(LFFoot, Ramp2, BodyHeight)

      PSC.SETPOS(RRKnee, Ramp2, BodyHeight)
      PSC.SETPOS(RRFoot, Ramp2, BodyHeight)  
       
        'Moves leg back and up
      PSC.SETPOS(LPawHip, Ramp, HipPositionLPaw)       
      PSC.SETPOS(LPawKnee, Ramp, LPawKneePW)
      PSC.SETPOS(LPawFoot, Ramp, FootPositionLPaw)  
      'waitcnt(clkfreq/10 + cnt)


MoveFootRF

PUB MoveFootRF
''Moves RFfoot to it's starting position
    Debug.str(string(13, "MoveFootRF")) 

    If FootPositionRF == FootEndpointRF
       FootLegStateRF  := 1
       Debug.str(string(13, "Foot RF is done moving; skipping RF to MoveFootLR! :) "))
       MoveFootLR
                              
    FootPositionRF := FootPositionRF + FootDirectionRF*HipSpeedMultiplier
    IF FootDirectionRF == 1
       FootPositionRF <#= FootEndpointRF
       Debug.str(string(13, "FootPositionRF <#= FootEndpointRF"))
    IF FootDirectionRF == -1
       FootPositionRF #>= FootEndpointRF
       Debug.str(string(13, "FootPositionRF #>= FootEndpointRF"))
       
    PSC.SETPOS(RFFoot, Ramp, FootPositionRF)
    'waitcnt(clkfreq/10 + cnt)
    
    ''This section checks to see if the Toe sensor is touching something and if it is,
    ''pulls the Foot back a bit and raises the leg
    If ina[RFToeSensor] == 1
      Debug.str(string(13, "Leg/pin RFToeSensor touching!!! :) "))
      FootPositionRF := FootPositionRF + (FootDirectionRF* -10)
      RFKneePW := RFKneePW + 15
      RFKneePW <#= KneeHightMax
      Debug.str(string(13, "RFKneePW "))
      Debug.dec(RFKneePW) 
     'Lift body a little bit
      BodyHeight := BodyHeight - 15     
      Debug.str(string(13, "BodyHeight"))
      Debug.dec(BodyHeight)
     
      If BodyHeight =< BodyHeightMax
           BodyHeight := BodyHeight -15
           Debug.str(string(13, "BackUpAndChangeApproach")) 'Going to back-up and change directions         
           BackUpAndChangeApproach 
      IF RFKneePW => KneeHightMax
           RFKneePW := RFKneePW - 15       
           Debug.str(string(13, "BackUpAndChangeApproach  due to RFKneePW")) 'Going to back-up and change directions         
           BackUpAndChangeApproach 
      'waitcnt(clkfreq/10 + cnt)
       
     'Lifting legs and raising body  
      Debug.str(string(13, "Raising Body"))
      PSC.SETPOS(RPawKnee, Ramp2, BodyHeight)
      PSC.SETPOS(RPawFoot, Ramp2, BodyHeight)

      PSC.SETPOS(LFKnee, Ramp2, BodyHeight)
      PSC.SETPOS(LFFoot, Ramp2, BodyHeight)

      PSC.SETPOS(RRKnee, Ramp2, BodyHeight)
      PSC.SETPOS(RRFoot, Ramp2, BodyHeight)  
         
      'Moves leg back and up
      PSC.SETPOS(RFHip, Ramp, HipPositionRF)       
      PSC.SETPOS(RFKnee, Ramp, RFKneePW)
      PSC.SETPOS(RFFoot, Ramp, FootPositionRF)  
      'waitcnt(clkfreq/10 + cnt)


MoveFootLR

PUB MoveFootLR
''Moves footLR to it's starting position
    Debug.str(string(13, "MoveFootLR")) 

    If FootPositionLR == FootEndpointLR
       FootLegStateLR  := 1
       Debug.str(string(13, "Foot LR is done moving; skipping LR to MoveFootGroup0! :) "))
       MoveFootGroup0

    FootPositionLR := FootPositionLR + FootDirectionLR*HipSpeedMultiplier
    IF FootDirectionLR == 1
       FootPositionLR <#= FootEndpointLR
       Debug.str(string(13, "FootPositionLR <#= FootEndpointLR"))
    IF FootDirectionLR == -1
       FootPositionLR #>= FootEndpointLR
       Debug.str(string(13, "FootPositionLR #>= FootEndpointLR"))
       
    PSC.SETPOS(LRFoot, Ramp, FootPositionLR)
    'waitcnt(clkfreq/10 + cnt)
    
    ''This section checks to see if the Toe sensor is touching something and if it is,
    ''pulls the Foot back a bit and raises the leg
    If ina[LRToeSensor] == 1
      Debug.str(string(13, "Leg/pin LRToeSensor touching!!! :) "))
      FootPositionLR := FootPositionLR + (FootDirectionLR* -10)
      LRKneePW := LRKneePW + 15
      LRKneePW <#= KneeHightMax
      Debug.str(string(13, "LRKneePW "))
      Debug.dec(LRKneePW) 
     'Lift body a little bit
      BodyHeight := BodyHeight - 15     
      Debug.str(string(13, "BodyHeight"))
      Debug.dec(BodyHeight)
     
      If BodyHeight =< BodyHeightMax
           BodyHeight := BodyHeight - 15
           Debug.str(string(13, "BackUpAndChangeApproach")) 'Going to back-up and change directions         
           BackUpAndChangeApproach 
      IF LRKneePW => KneeHightMax
           LRKneePW := LRKneePW - 15 
           Debug.str(string(13, "BackUpAndChangeApproach due to LRKneePW")) 'Going to back-up and change directions         
           BackUpAndChangeApproach 
      'waitcnt(clkfreq/10 + cnt)
       
     'Lifting legs and raising body  
      Debug.str(string(13, "Raising Body"))
      PSC.SETPOS(RPawKnee, Ramp2, BodyHeight)
      PSC.SETPOS(RPawFoot, Ramp2, BodyHeight)

      PSC.SETPOS(LFKnee, Ramp2, BodyHeight)
      PSC.SETPOS(LFFoot, Ramp2, BodyHeight)

      PSC.SETPOS(RRKnee, Ramp2, BodyHeight)
      PSC.SETPOS(RRFoot, Ramp2, BodyHeight)  
        
      'Moves leg back and up
      PSC.SETPOS(LRHip, Ramp, HipDirectionLR)       
      PSC.SETPOS(LRKnee, Ramp, LRKneePW)
      PSC.SETPOS(LRFoot, Ramp, FootPositionLR)  
      'waitcnt(clkfreq/10 + cnt)

MoveFootGroup0

PUB SetFootLowerFootStatesGroup0

''resets the leg states of the feet to zero so that they can be used again
''for seeing if all the feet are touching the ground
    Debug.str(string(13, "SetFootLowerFootStatesGroup0")) 


FootState0       :=  0
FootLegStateLPaw :=  0
FootLegStateRF   :=  0
FootLegStateLR   :=  0
 
LowerFootGroup0

PUB LowerFootGroup0
''Lowers feet until they touch something
    Debug.str(string(13, "LowerFootGroup0")) 

'lowers feet until they all touch ground
'also lowers the body, if it's higher than 750 (middle)

    Debug.str(string(13, "FootLegStateLPaw "))
    Debug.dec(FootLegStateLPaw)
    Debug.str(string(13, "FootLegStateRF "))
    Debug.dec(FootLegStateRF)
    Debug.str(string(13, "FootLegStateLR "))         
    Debug.dec(FootLegStateLR)
    Debug.str(string(13, "FootState0 "))
    Debug.dec(FootState0)

 
    FootState0 := FootLegStateLPaw + FootLegStateRF + FootLegStateLR
    FootState0 := 3      '+=+=+=+************************ for debugging only)
                         'this way it thinks it's feet are touching so that I don't have to have it on the ground durring debugging (it freezes without this as well)
    If FootState0 == 3                               '+=+=+=+***** change "1" to "3" (1 used for debugging)
       Debug.str(string(13, "Foot group 0 are down and in place; now we can go and lift up other legs and move forward :) ")) 
       MoveBodyForwardViaGroup0

LowerFootRPaw


PUB LowerFootRPaw
''Lowers foot RPaw
     Debug.str(string(13, "LowerFootRPaw"))


   If ina[LPawToeSensor] == 1
       FootLegStateLPaw  := 1
       Debug.str(string(13, "Foot LPaw is done lowering; skipping LPaw to LowerFootRF! :) "))
       LowerFootRf
    'lower body
   ELSEIF BodyHeight => BodyHeightMin
      BodyHeight := BodyHeight + 15
   
    'lower foot
   ELSEIF LPawKneePW =< KneeHightMin
       Debug.str(string(13, "Foot LPaw can't touch ground; going in other direction BackUpAndChangeApproach "))
       BackUpAndChangeApproach
      
   LPawKneePW := LPawKneePW - 15
   Debug.str(string(13, "LPawKneePW "))
   Debug.dec(LPawKneePW) 
  'Lowers body a little bit
   BodyHeight := BodyHeight + 15     
   Debug.str(string(13, "BodyHeight"))
   Debug.dec(BodyHeight)

   If BodyHeight => BodyHeightMin
      Debug.str(string(13, "BackUpAndChangeApproach")) 'Going to back-up and change directions         
      BackUpAndChangeApproach 
   IF LPawKneePW =< KneeHightMin           
      Debug.str(string(13, "BackUpAndChangeApproach due to LPawKneePW min")) 'Going to back-up and change directions         
      BackUpAndChangeApproach 
 

  'Lowers legs and lowers body  
   Debug.str(string(13, "Lowers Body"))
   PSC.SETPOS(RPawKnee, Ramp2, BodyHeight)
   PSC.SETPOS(RPawFoot, Ramp2, BodyHeight)

   PSC.SETPOS(LFKnee, Ramp2, BodyHeight)
   PSC.SETPOS(LFFoot, Ramp2, BodyHeight)

   PSC.SETPOS(RRKnee, Ramp2, BodyHeight)
   PSC.SETPOS(RRFoot, Ramp2, BodyHeight)  
        
  'Moves leg down
   PSC.SETPOS(LPawHip, Ramp, HipPositionLPaw)       
   PSC.SETPOS(LPawKnee, Ramp, LPawKneePW)
   PSC.SETPOS(LPawFoot, Ramp, FootPositionLPaw)  
    'waitcnt(clkfreq/10 + cnt)
                       
LowerFootRf


PUB LowerFootRf
''Lowers foot RF
     Debug.str(string(13, "LowerFootRf"))

   If ina[RFToeSensor] == 1
       FootLegStateRF  := 1
       Debug.str(string(13, "Foot RF is done moving; skipping RF to LowerFootLR! :) "))
       LowerFootLR
    'lower body
   ELSEIF BodyHeight => BodyHeightMin
      BodyHeight := BodyHeight + 15
   
    'lower foot
   ELSEIF RFKneePW =< KneeHightMin
       Debug.str(string(13, "Foot Y can't touch ground; going in other direction BackUpAndChangeApproach "))
       BackUpAndChangeApproach
      
   RFKneePW := RFKneePW - 15
   Debug.str(string(13, "RFKneePW "))
   Debug.dec(RFKneePW) 
  'Lowers body a little bit
   BodyHeight := BodyHeight + 15     
   Debug.str(string(13, "BodyHeight"))
   Debug.dec(BodyHeight)

   If BodyHeight => BodyHeightMin
      Debug.str(string(13, "BackUpAndChangeApproach")) 'Going to back-up and change directions         
      BackUpAndChangeApproach 
   IF RFKneePW =< KneeHightMin           
      Debug.str(string(13, "BackUpAndChangeApproach due to RFKneePW")) 'Going to back-up and change directions         
      BackUpAndChangeApproach 
 

  'Lowers legs and lowers body  
   Debug.str(string(13, "Lowers Body"))
   PSC.SETPOS(RPawKnee, Ramp2, BodyHeight)
   PSC.SETPOS(RPawFoot, Ramp2, BodyHeight)

   PSC.SETPOS(LFKnee, Ramp2, BodyHeight)
   PSC.SETPOS(LFFoot, Ramp2, BodyHeight)

   PSC.SETPOS(RRKnee, Ramp2, BodyHeight)
   PSC.SETPOS(RRFoot, Ramp2, BodyHeight)  

  'Moves leg down
   PSC.SETPOS(RFHip, Ramp, HipPositionRF)       
   PSC.SETPOS(RFKnee, Ramp, RFKneePW)
   PSC.SETPOS(RFFoot, Ramp, FootPositionRF)  
    'waitcnt(clkfreq/10 + cnt)


LowerFootLR

PUB LowerFootLR
''Lowers foot LR
     Debug.str(string(13, "LowerFootLR"))

   If ina[LRToeSensor] == 1
       FootLegStateLR  := 1
       Debug.str(string(13, "Foot LR is done moving; skipping LR to LowerFootGroup0! :) "))
       LowerFootGroup0
    'lower body
   ELSEIF BodyHeight => BodyHeightMin
      BodyHeight := BodyHeight + 15
   
    'lower foot
   ELSEIF LRKneePW =< KneeHightMin
       Debug.str(string(13, "Foot LR can't touch ground; going in other direction BackUpAndChangeApproach "))
       BackUpAndChangeApproach
      
   LRKneePW := LRKneePW - 15
   Debug.str(string(13, "LRKneePW "))
   Debug.dec(LRKneePW) 
  'Lowers body a little bit
   BodyHeight := BodyHeight + 15     
   Debug.str(string(13, "BodyHeight"))
   Debug.dec(BodyHeight)

   If BodyHeight => BodyHeightMin
      Debug.str(string(13, "BackUpAndChangeApproach")) 'Going to back-up and change directions         
      BackUpAndChangeApproach 
   IF LRKneePW =< KneeHightMin           
      Debug.str(string(13, "BackUpAndChangeApproach due to LRKneePW")) 'Going to back-up and change directions         
      BackUpAndChangeApproach 
 


   Debug.str(string(13, "Lowers Body"))
   PSC.SETPOS(RPawKnee, Ramp2, BodyHeight)
   PSC.SETPOS(RPawFoot, Ramp2, BodyHeight)

   PSC.SETPOS(LFKnee, Ramp2, BodyHeight)
   PSC.SETPOS(LFFoot, Ramp2, BodyHeight)

   PSC.SETPOS(RRKnee, Ramp2, BodyHeight)
   PSC.SETPOS(RRFoot, Ramp2, BodyHeight)  

 'Moves leg down
   PSC.SETPOS(LRHip, Ramp, HipPositionLR)       
   PSC.SETPOS(LRKnee, Ramp, LRKneePW)
   PSC.SETPOS(LRFoot, Ramp, FootPositionLR)  
    'waitcnt(clkfreq/10 + cnt)
  


LowerFootGroup0







PUB MoveBodyForwardViaGroup0
    Debug.str(string(13, "MoveBodyForwardViaGroup0 "))

'Lift non moving legs off the ground

   'BodyHeight := BodyHeight + 150                   
   Debug.str(string(13, "BodyHeight"))
   Debug.dec(BodyHeight)


  'Lifts non moving legs 
   Debug.str(string(13, "Lifts non moving legs"))
   PSC.SETPOS(RPawKnee, Ramp2, BodyHeight)
   PSC.SETPOS(RPawFoot, Ramp2, BodyHeight)

   PSC.SETPOS(LFKnee, Ramp2, BodyHeight)
   PSC.SETPOS(LFFoot, Ramp2, BodyHeight)

   PSC.SETPOS(RRKnee, Ramp2, BodyHeight)
   PSC.SETPOS(RRFoot, Ramp2, BodyHeight)  
 
 waitcnt(clkfreq*1 + cnt)

'Down legs pull body forward
   Debug.str(string(13, "Down legs pull body forward")) 
   PSC.SETPOS(LPawHip, Ramp, HipStartPointLPaw)       
   PSC.SETPOS(LPawKnee, Ramp, LPawKneePW)
   PSC.SETPOS(LPawFoot, Ramp, FootStartPointLPaw)  
 
   PSC.SETPOS(RFHip, Ramp, HipStartPointRF)       
   PSC.SETPOS(RFKnee, Ramp, RFKneePW)
   PSC.SETPOS(RFFoot, Ramp, FootStartPointRF)  
 
   PSC.SETPOS(LRHip, Ramp, HipStartPointLR)       
   PSC.SETPOS(LRKnee, Ramp, LRKneePW)
   PSC.SETPOS(LRFoot, Ramp, FootStartPointLR)  

   Debug.str(string(13, "Legs moved?")) 

   waitcnt(clkfreq*4 + cnt)

LegValues

   

PUB  Group1    
    Debug.str(string(13, "Group 1 "))
    'empty for right now until I get this freezing bug fixed


  'waitcnt(clkfreq*10 + cnt)


PUB BackUpAndChangeApproach
    Debug.str(string(13, "BackUpAndChangeApproach "))
    'empty for right now until I get this freezing bug fixed









PUB CaculateLegValues(MasterDirection, LegNumber, TotalAmountOfMovementInOneTurn) | CorrectedLegAngle, WalkingDirection, CorrectedLegAngle0to90, HipDirection, FootDirection, PercentFootmovement, TotalFootMovement,  HipMidpoint, FootMidpoint
'HipStartpoint, HipEndPoint, FootStartpoint, FootEndPoint  <-- these are global variables
Debug.str(string(13, "CaculateLegValues "))
F.Start 
    Debug.str(string(13, "MasterDirection "))
    Debug.dec(MasterDirection)

    Debug.str(string(13, "LegNumber "))
    Debug.dec(LegNumber)




'Caculates the corrected angle that the particulare leg needs inorder for all the legs to move in the correct direction
    CorrectedLegAngle := MasterDirection + 60 * LegNumber                               '60 degrees is what the AH-R3 needs
    'WalkingDirection  := CorrectedLegAngle

    Debug.str(string(13, "CorrectedLegAngle "))
    Debug.dec(CorrectedLegAngle)

'Caculates what direction the legs and hips move based on the corrected leg Angle
    IF CorrectedLegAngle => 360
       CorrectedLegAngle := CorrectedLegAngle - 360

    'IF     CorrectedLegAngle => 360
    '        HipDirection  :=  1
     '       FootDirection :=  1                   
     '       CorrectedLegAngle := CorrectedLegAngle - 360
    IF  CorrectedLegAngle => 270
            HipDirection  := -1
            FootDirection :=  1                   
    ELSEIF CorrectedLegAngle => 180  
            HipDirection  :=  1
            FootDirection :=  1
    ELSEIF CorrectedLegAngle =>  90  
            HipDirection  :=  1
            FootDirection := -1
    ELSE   
            HipDirection  := -1
            FootDirection := -1

'Makes the angle between 0 and 90 for caculating how much movement is done by the foot or hip
    CorrectedLegAngle0to90 := CorrectedLegAngle

    IF CorrectedLegAngle0to90 => 181
       CorrectedLegAngle0to90 := CorrectedLegAngle0to90 - 180

    IF CorrectedLegAngle0to90 => 91
       CorrectedLegAngle0to90 := CorrectedLegAngle0to90 - 90
       PW                     := CorrectedLegAngle0to90
       CorrectedLegAngle0to90 := 90 - PW
       'need to do this so that, ex. 92 degrees doesn't become 2 degrees instead of 88 degrees like it should
   
    Debug.str(string(13, "CorrectedLegAngle0to90 "))
    Debug.dec(CorrectedLegAngle0to90)

    CorrectedLegAngle0to90         := f.FFloat(CorrectedLegAngle0to90)
    TotalAmountOfMovementInOneTurn := f.FFloat(TotalAmountOfMovementInOneTurn)
    TotalFootMovement              := f.FFloat(TotalFootMovement)
    
    PercentFootmovement := f.fDiv(CorrectedLegAngle0to90, 90.0)
    TotalFootMovement   := f.fMul(TotalAmountOfMovementInOneTurn, PercentFootmovement)
    TotalHipMovement    := f.FSub(TotalAmountOfMovementInOneTurn, TotalFootMovement)



    'Debug.str(string(13, "TotalFootMovement "))
    'Debug.dec(TotalFootMovement)
    'Debug.str(string(13, "TotalHipMovement "))
    'Debug.dec(TotalHipMovement)

   Debug.Str(String(13, "TotalFootMovement float"))
   Debug.str(fstring.FloatToString(TotalFootMovement))
   Debug.Str(String(13, "TotalHipMovement float"))
   Debug.str(fstring.FloatToString(TotalHipMovement))


   TotalFootMovement := f.FRound(TotalFootMovement)
   Debug.str(string(13, "TotalFootMovement  Round"))
   Debug.dec(TotalFootMovement)


   TotalHipMovement := f.FRound(TotalHipMovement)
   Debug.str(string(13, "TotalHipMovement Round"))
   Debug.dec(TotalHipMovement)




    Debug.str(string(13, "TotalFootMovement "))
    Debug.dec(TotalFootMovement)
    Debug.str(string(13, "TotalHipMovement "))
    Debug.dec(TotalHipMovement)

    HipMidpoint   := 750
    HipStartpoint   := HipMidpoint + (TotalHipMovement/2)*HipDirection  '+  '-
    HipEndPoint := HipMidpoint - (TotalHipMovement/2)*HipDirection  '-  '+

    Debug.str(string(13, "HipStartpoint "))
    Debug.dec(HipStartpoint)
    Debug.str(string(13, "HipEndPoint "))
    Debug.dec(HipEndPoint)

    FootMidpoint   := 750
    FootStartpoint   := FootMidpoint + (TotalFootMovement/2)*FootDirection '+ '-
    FootEndPoint := FootMidpoint - (TotalFootMovement/2)*FootDirection '-  '+

    Debug.str(string(13, "FootStartpoint "))
    Debug.dec(FootStartpoint)
    Debug.str(string(13, "FootEndPoint "))
    Debug.dec(FootEndPoint)

'  waitcnt(clkfreq*66 + cnt)        '<--- for debugging and allowing time to check values******************
  

F.Stop



    